@@ -5,6 +5,7 @@ use std::fmt;
 use std::thread;
 use std::time::Instant;
 
+use itertools::Itertools;
 use prost_types::Any;
 use thiserror::Error;
 use tracing::{debug, error, info, trace, warn};
@@ -194,8 +195,10 @@ impl fmt::Display for OperationalData {
 
 pub struct RelayPath {
     channel: Channel,
+    // Marks whether this path has already cleared pending packets.
+    // Packets should be cleared once (at startup), then this
+    // flag turns to `false`.
     clear_packets: bool,
-
     // Operational data, targeting both the source and destination chain.
     // These vectors of operational data are ordered decreasingly by their age, with element at
     // position `0` being the oldest.
@@ -397,7 +400,6 @@ impl RelayPath {
     }
 
     fn relay_pending_packets(&mut self, height: Height) -> Result<(), LinkError> {
-        info!("[{}] clearing old packets", self);
         for _ in 0..MAX_RETRIES {
             if self
                 .build_recv_packet_and_timeout_msgs(Some(height))
@@ -410,30 +412,45 @@ impl RelayPath {
         Err(LinkError::OldPacketClearingFailed)
     }
 
-    /// Should not run more than once per execution.
+    /// Queries the source chain at the given [`Height`]
+    /// to find any packets or acknowledgements that are pending,
+    /// and fetches the relevant packet event data. Finally, this
+    /// method also schedules the corresponding operational data,
+    /// so that the relayer will later relay the pending packets.
     pub fn clear_packets(&mut self, above_height: Height) -> Result<(), LinkError> {
-        if self.clear_packets {
-            info!(
-                "[{}] clearing pending packets from events before height {}",
-                self, above_height
-            );
+        info!(
+            "[{}] clearing pending packets from events before height {}",
+            self, above_height
+        );
 
-            let clear_height = above_height.decrement().map_err(|e| LinkError::Failed(
-                format!("Cannot clear packets @height {}, because this height cannot be decremented: {}", above_height, e.to_string())))?;
+        let clear_height = above_height.decrement().map_err(|e| {
+            LinkError::Failed(format!(
+                "Cannot clear packets @height {}, because this height cannot be decremented: {}",
+                above_height,
+                e.to_string()
+            ))
+        })?;
 
-            self.relay_pending_packets(clear_height)?;
+        self.relay_pending_packets(clear_height)?;
 
-            info!("[{}] finished clearing pending packets", self);
-
-            self.clear_packets = false;
-        }
+        info!(
+            "[{}] finished scheduling the clearing of pending packets",
+            self
+        );
 
         Ok(())
     }
 
     /// Generate & schedule operational data from the input `batch` of IBC events.
     pub fn update_schedule(&mut self, batch: EventBatch) -> Result<(), LinkError> {
-        self.clear_packets(batch.height)?;
+        // With the first batch of events, also trigger the clearing of old packets.
+        if self.clear_packets {
+            self.clear_packets(batch.height)?;
+
+            // Disable further clearing of old packet.
+            // Clearing will happen separately, upon new blocks.
+            self.clear_packets = false;
+        }
 
         // Collect relevant events from the incoming batch & adjust their height.
         let events = self.filter_events(&batch.events);
@@ -598,10 +615,11 @@ impl RelayPath {
 
         for i in 0..MAX_RETRIES {
             info!(
-                "[{}] relay op. data to {}, proofs height {}, (delayed by: {:?}) [try {}/{}]",
+                "[{}] relay op. data of {} msgs(s) to {} (height {}), delayed by: {:?} [try {}/{}]",
                 self,
+                odata.batch.len(),
                 odata.target,
-                odata.proofs_height,
+                odata.proofs_height.increment(),
                 odata.scheduled_time.elapsed(),
                 i + 1,
                 MAX_RETRIES
@@ -721,7 +739,8 @@ impl RelayPath {
     }
 
     /// Sends a transaction to the chain targeted by the operational data `odata`.
-    /// If the transaction generates an error, returns the error as well as  `LinkError::SendError` if  input events if a sending failure occurs.
+    /// If the transaction generates an error, returns the error as well as  `LinkError::SendError`
+    /// if  input events if a sending failure occurs.
     /// Returns the events generated by the target chain upon success.
     fn send_from_operational_data(
         &mut self,
@@ -921,19 +940,13 @@ impl RelayPath {
         if packet_commitments.is_empty() {
             return Ok((events_result, query_height));
         }
-        let commit_sequences = packet_commitments.iter().map(|p| p.sequence).collect();
-        debug!(
-            "[{}] packets that still have commitments on {}: {:?}",
-            self,
-            self.src_chain().id(),
-            commit_sequences
-        );
+        let commit_sequences: Vec<u64> = packet_commitments.iter().map(|p| p.sequence).collect();
 
         // Get the packets that have not been received on destination chain
         let request = QueryUnreceivedPacketsRequest {
             port_id: self.dst_port_id().to_string(),
             channel_id: self.dst_channel_id()?.to_string(),
-            packet_commitment_sequences: commit_sequences,
+            packet_commitment_sequences: commit_sequences.clone(),
         };
 
         let sequences: Vec<Sequence> = self
@@ -943,18 +956,26 @@ impl RelayPath {
             .map(From::from)
             .collect();
 
-        debug!(
-            "[{}] recv packets to send out to {} of the ones with commitments on source {}: {:?}",
-            self,
-            self.dst_chain().id(),
-            self.src_chain().id(),
-            sequences
-        );
-
         if sequences.is_empty() {
             return Ok((events_result, query_height));
         }
 
+        debug!(
+            "[{}] packets that still have commitments on {}: {} (first 10 shown here; total={})",
+            self,
+            self.src_chain().id(),
+            commit_sequences.iter().take(10).join(", "),
+            commit_sequences.len()
+        );
+
+        debug!(
+            "[{}] recv packets to send out to {} of the ones with commitments on source {}: {} (first 10 shown here; total={})",
+            self,
+            self.dst_chain().id(),
+            self.src_chain().id(),
+            sequences.iter().take(10).join(", "), sequences.len()
+        );
+
         let query = QueryTxRequest::Packet(QueryPacketEventDataRequest {
             event_id: IbcEventType::SendPacket,
             source_port_id: self.src_port_id().clone(),
@@ -972,8 +993,17 @@ impl RelayPath {
             let send_event = downcast!(event => IbcEvent::SendPacket)
                 .ok_or_else(|| LinkError::Failed("unexpected query tx response".into()))?;
             packet_sequences.push(send_event.packet.sequence);
+            if packet_sequences.len() > 10 {
+                // Enough to print the first 10
+                break;
+            }
         }
-        debug!("[{}] received from query_txs {:?}", self, packet_sequences);
+        info!(
+            "[{}] found unprocessed SendPacket events for {:?} (first 10 shown here; total={})",
+            self,
+            packet_sequences,
+            events_result.len()
+        );
 
         Ok((events_result, query_height))
     }
@@ -1006,18 +1036,13 @@ impl RelayPath {
             return Ok((events_result, query_height));
         }
 
-        let acked_sequences = acks_on_source.iter().map(|p| p.sequence).collect();
-        debug!(
-            "[{}] packets that have acknowledgments on {} {:?}",
-            self,
-            self.src_chain().id(),
-            acked_sequences
-        );
+        let mut acked_sequences: Vec<u64> = acks_on_source.iter().map(|p| p.sequence).collect();
+        acked_sequences.sort_unstable();
 
         let request = QueryUnreceivedAcksRequest {
             port_id: self.dst_port_id().to_string(),
             channel_id: dst_channel_id.to_string(),
-            packet_ack_sequences: acked_sequences,
+            packet_ack_sequences: acked_sequences.clone(),
         };
 
         let sequences: Vec<Sequence> = self
@@ -1027,18 +1052,28 @@ impl RelayPath {
             .into_iter()
             .map(From::from)
             .collect();
-        debug!(
-            "[{}] ack packets to send out to {} of the ones with acknowledgments on {}: {:?}",
-            self,
-            self.dst_chain().id(),
-            self.src_chain().id(),
-            sequences
-        );
 
         if sequences.is_empty() {
             return Ok((events_result, query_height));
         }
 
+        debug!(
+            "[{}] packets that have acknowledgments on {}: [{:?}..{:?}] (total={})",
+            self,
+            self.src_chain().id(),
+            acked_sequences.first(),
+            acked_sequences.last(),
+            acked_sequences.len()
+        );
+
+        debug!(
+            "[{}] ack packets to send out to {} of the ones with acknowledgments on {}: {} (first 10 shown here; total={})",
+            self,
+            self.dst_chain().id(),
+            self.src_chain().id(),
+            sequences.iter().take(10).join(", "), sequences.len()
+        );
+
         events_result = self
             .src_chain()
             .query_txs(QueryTxRequest::Packet(QueryPacketEventDataRequest {
@@ -1057,8 +1092,12 @@ impl RelayPath {
             let write_ack_event = downcast!(event => IbcEvent::WriteAcknowledgement)
                 .ok_or_else(|| LinkError::Failed("unexpected query tx response".into()))?;
             packet_sequences.push(write_ack_event.packet.sequence);
+            if packet_sequences.len() > 10 {
+                // Enough to print the first 10
+                break;
+            }
         }
-        info!("[{}] received from query_txs {:?}", self, packet_sequences);
+        info!("[{}] found unprocessed WriteAcknowledgement events for {:?} (first 10 shown here; total={})", self, packet_sequences, events_result.len());
 
         Ok((events_result, query_height))
     }
@@ -1326,11 +1365,17 @@ impl RelayPath {
                     IbcEvent::SendPacket(e) => {
                         // Catch any SendPacket event that timed-out
                         if self.send_packet_event_handled(e)? {
-                            debug!("[{}] {} already handled", self, e);
+                            debug!(
+                                "[{}] refreshing schedule: already handled send packet {}",
+                                self, e
+                            );
                         } else if let Some(new_msg) =
                             self.build_timeout_from_send_packet_event(e, dst_current_height)?
                         {
-                            debug!("[{}] found a timed-out msg in the op data {}", self, odata);
+                            debug!(
+                                "[{}] refreshing schedule: found a timed-out msg in the op data {}",
+                                self, odata
+                            );
                             timed_out.entry(odata_pos).or_insert_with(Vec::new).push(
                                 TransitMessage {
                                     event: event.clone(),
@@ -1344,7 +1389,10 @@ impl RelayPath {
                     }
                     IbcEvent::WriteAcknowledgement(e) => {
                         if self.write_ack_event_handled(e)? {
-                            debug!("[{}] {} already handled", self, e);
+                            debug!(
+                                "[{}] refreshing schedule: already handled {} write ack ",
+                                self, e
+                            );
                         } else {
                             retain_batch.push(gm.clone());
                         }
@@ -1357,6 +1405,13 @@ impl RelayPath {
             odata.batch = retain_batch;
         }
 
+        // Replace the original operational data with the updated one
+        self.dst_operational_data = all_dst_odata;
+        // Possibly some op. data became empty (if no events were kept).
+        // Retain only the non-empty ones.
+        self.dst_operational_data.retain(|o| !o.batch.is_empty());
+
+        // Handle timed-out events
         if timed_out.is_empty() {
             // Nothing timed out in the meantime
             return Ok(());
@@ -1370,7 +1425,7 @@ impl RelayPath {
             new_od.batch = batch;
 
             info!(
-                "[{}] re-scheduling from new timed-out batch of size {}",
+                "[{}] refreshing schedule: re-scheduling from new timed-out batch of size {}",
                 self,
                 new_od.batch.len()
             );
@@ -1378,12 +1433,6 @@ impl RelayPath {
             self.schedule_operational_data(new_od)?;
         }
 
-        self.dst_operational_data = all_dst_odata;
-
-        // Possibly some op. data became empty (if no events were kept).
-        // Retain only the non-empty ones.
-        self.dst_operational_data.retain(|o| !o.batch.is_empty());
-
         Ok(())
     }
 
@@ -1400,7 +1449,7 @@ impl RelayPath {
         }
 
         info!(
-            "[{}] scheduling op. data with {} msg(s) for {} chain (height {})",
+            "[{}] scheduling op. data with {} msg(s) for {} (height {})",
             self,
             od.batch.len(),
             od.target,
