@@ -1,17 +1,18 @@
+use std::time::Duration;
+
 use thiserror::Error;
 use tracing::error;
 
 use ibc::application::ics20_fungible_token_transfer::msgs::transfer::MsgTransfer;
 use ibc::events::IbcEvent;
 use ibc::ics24_host::identifier::{ChainId, ChannelId, PortId};
+use ibc::timestamp::Timestamp;
 use ibc::tx_msg::Msg;
+use ibc::Height;
 
-use crate::chain::{Chain, CosmosSdkChain};
+use crate::chain::handle::ChainHandle;
 use crate::config::ChainConfig;
 use crate::error::Error;
-use ibc::timestamp::Timestamp;
-use ibc::Height;
-use std::time::Duration;
 
 #[derive(Debug, Error)]
 pub enum PacketError {
@@ -22,7 +23,7 @@ pub enum PacketError {
     Key(Error),
 
     #[error(
-        "failed during a transaction submission step to chain id {0} with underlying error: {1}"
+        "failed while submitting the Transfer message to chain {0} with underlying error: {1}"
     )]
     Submit(ChainId, Error),
 
@@ -45,8 +46,8 @@ pub struct TransferOptions {
 }
 
 pub fn build_and_send_transfer_messages(
-    mut packet_src_chain: CosmosSdkChain, // the chain whose account is debited
-    mut packet_dst_chain: CosmosSdkChain, // the chain where the transfer is sent
+    packet_src_chain: Box<dyn ChainHandle>, // the chain whose account is debited
+    packet_dst_chain: Box<dyn ChainHandle>, // the chain whose account eventually gets credited
     opts: TransferOptions,
 ) -> Result<Vec<IbcEvent>, PacketError> {
     let receiver = match &opts.receiver {
@@ -90,7 +91,7 @@ pub fn build_and_send_transfer_messages(
 
     let events = packet_src_chain
         .send_msgs(msgs)
-        .map_err(|e| PacketError::Submit(packet_src_chain.id().clone(), e))?;
+        .map_err(|e| PacketError::Submit(packet_src_chain.id(), e))?;
 
     // Check if the chain rejected the transaction
     let result = events
