@@ -7,6 +7,10 @@ use crate::event::monitor::EventBatch;
 pub enum WorkerCmd {
     /// A batch of packet events need to be relayed
     IbcEvents { batch: EventBatch },
+
     /// A batch of [`NewBlock`] events need to be relayed
     NewBlock { height: Height, new_block: NewBlock },
+
+    /// Shutdown the worker
+    Shutdown,
 }
